////////////////////////////////////////////////////////////////////////////////

//   VoidArena

//

//   Replaces all game weapons/ammo with custom weapon/ammo, and changes the

//   default weapon from the impact hammer to the translocator.

//

//   This code combines elements of the standard Arena mutator with common

//   weapon swapper code found in most zark and clan sniper rifles, which (to my

//   knowledge) was created by SpawnKiller/SkullKrusher.

//

//   Authored by :[lol]:Mhor of www.teamlol.com

//

////////////////////////////////////////////////////////////////////////////////

class VoidArena extends Mutator;



var   name   WeaponName,         //The name of our custom weapon.

         AmmoName;         //The name of our custom ammo.

var   string   WeaponString,      //The string Package.Class name of our custom weapon.

         AmmoString;         //The string Package.Class name of our custom ammo.



//Set the level's default weapon prior to gameplay.

event PreBeginPlay()

{

   Level.Game.DefaultWeapon=class'VoidSniperRifleV3.VoidXTrans';

}



//Give our custom weapon to the player upon spawn. Note that we don't set the

//game's default weapon to the custom weapon. This is because default weapon

//is not "tossed" as a pickup when the player dies, and we like those pickups

//so that other players can reload using a dead player's gun.

function ModifyPlayer(Pawn Other)

{

   //Give the player our custom weapon.

   DeathMatchPlus(Level.Game).GiveWeapon(Other,WeaponString);



   //Never break the chain, baby.

   if(NextMutator != None)

      NextMutator.ModifyPlayer(Other);

}



//This function ensures that the custom weapons & ammo that are spawned into

//the game are "always kept." This means that CheckReplacement is not even

//called if AlwaysKeep comes back as true from the mutator chain. Typically, the

//mutators are coded so that if any one mutator wants to "keep" the actor, the

//rest of the mutators aren't consulted, the true value is returned immediately,

//and the object is spawned into the game.

function bool AlwaysKeep(Actor Other)

{

   //We don't want enforcers/hammers at any time, so return false before

   //any other mutators in the chain return true (and bypass CheckReplacement).

   //This won't guarantee that the item isn't spawned, but it will try to make

   //sure that any other mutators can't force the item to be spawned into the

   //game. This is used in tandem with CheckReplacement() below.

   //Note: these two weapons are somewhat special. The enforcer is hard coded

   //into DeathMatchPlus to be given to every player, and the impact hammer is

   //usually a default game weapon. Performing a ReplaceWith() on either of

   //these weapons while being given to the player (DMP code or default weapon),

   //you get the problem of weapons piling up at spawn points. If you've got

   //your default weapon set up properly, and no other mutators or game settings

   //aren't going to spawn impact hammers as default weapons, you can get away

   //with just the enforcer statement below. And if you want the impact hammer

   //in the game (default weapon or otherwise), then you MUST remove it from

   //the statement below, and the corresponding statement in CheckReplacement.

   if(Other.IsA('Enforcer') || Other.IsA('ImpactHammer'))

      return false;



   //If our custom weapon is being spawned...

   if(Other.IsA(WeaponName))

   {

      //Set the pickup ammo amount to the weapon's default ammo ammount. Makes

      //all weapons behave as pickup items.

      Weapon(Other).PickupAmmoCount=Weapon(Other).Default.PickUpAmmoCount;

      //Returning true means the object will be spawned in game, and IsRelevant

      //and CheckReplacement won't be called to tamper with the spawn.

      return true;

   }

   //If our custom ammo is being spawned...

   if(Other.IsA(AmmoName))

   {

      //Set the pickup ammo amount to the ammo's default ammo ammount.

      Ammo(Other).AmmoAmount=Ammo(Other).AmmoAmount;

      //Returning true means the object will be spawned in game, and IsRelevant

      //and CheckReplacement won't be called to tamper with the spawn.

      return true;

   }



   //If the object is not a custom weapon/ammo, then consult the next mutator.

   if(NextMutator != None)

      return(NextMutator.AlwaysKeep(Other));

   //If the object is not a custom weapon/ammo, and this is the last mutator,

   //then return false, which means that IsRelevant and CheckReplacement will

   //be consulted to determine if the object should be spawned.

   return false;

}



//This function allows the mutator to selectively replace certain objects with

//others, such as weapon and inventory swappers. If an object is to be replaced,

//a false value is returned to IsRelevant, which means the object is not spawned.

//At the same time, ReplaceWith is called which starts the spawn process for the

//replacement object, and makes sure that necessary object properties are

//transferred from the replaced object to the replacement object. bSuperRelevant

//deals with whether game logic about odds of appearing, game goals, etc. have

//control over the object spawn - if bSuperRelevant is "1", then this game logic

//is bypassed. Unless a specific rational/objective exists, there should be no

//reason to set bSuperRelevant to "1" in the mutator.

//Note: we avoid recursive errors in CheckReplacement (due to the way "IsA()"

//works) by returning true in AlwaysKeep() above for our replacement objects

//(which means IsRelevant and CheckReplacement code won't be called for the

//replacement objects). But you'll see below that a last-minute check is

//performed just before the ReplaceWith function is called.

//Note: when false is returned from CheckReplacement, the checks through the

//mutator chain's IsRelevant and CheckReplacement functions are halted, and

//false returned to GameInfo. This causes the object to be destroyed from the

//game without further checks.

function bool CheckReplacement(Actor Other, out byte bSuperRelevant)

{

   //If a weapon is being spawned...

   if (Other.IsA('Weapon'))

   {

      //Make sure that enforcers and impact hammers aren't spawned into the

      //level. This is working in tandem with the AlwaysKeep() code above that

      //pertains to enforcers/hammers. Remember, if you want impact hammers

      //in game, edit the following line (see AlwaysKeep above).

      if(Other.IsA('Enforcer') || Other.IsA('ImpactHammer'))

         return false;



      //If the weapon is the default weapon then return true (all other

      //mutators have their shot at replacing/removing the default weapon).

      //If we use AlwaysKeep to return true above, similar to the custom

      //weapon/ammo, it can cause problems with games set up to remove

      //certain default weapons (like the translocator).

      else if(Other.IsA('Translocator'))

         return true;



      //Else go through the replacement code...

      else

      {

         //If the weaponstring variable has not been correctly set, don't spawn

         //the replacement or the original object, and exit immediately.

         if(WeaponString=="")

            return false;

         //A second check to make sure this weapon isn't our custom weapon, which

         //would cause a recursive error. This should never happen with the

         //AlwaysKeep code put in place above, but it never hurts to be safe.

         else if (!Other.IsA(WeaponName))

         {

            //Since the object is definitely not our custom weapon, replace it

            //with the custom weapon, and return false so that the original

            //object is not spawned.

            ReplaceWith(Other,WeaponString);

            return false;

         }

      }

   }

   //If a ammo is being spawned...

   if(Other.IsA('Ammo'))

   {

      //If the ammostring variable has not been correctly set, don't spawn

      //the replacement or the original object, and exit immediately.

      if(AmmoString=="")

         return false;

      //A second check to make sure this ammo isn't our custom ammo, which

      //would cause a recursive error. This should never happen with the

      //AlwaysKeep code put in place above, but it never hurts to be safe.

      else if (!Other.IsA(AmmoName))

      {

         //Since the object is definitely not our custom ammo, replace it

         //with the custom ammo, and return false so that the original

         //object is not spawned.

         ReplaceWith(Other,AmmoString);

         return false;

      }

   }



   //If the object isn't a custom weapon or custom ammo, then return true. This

   //will cause the checks through the mutator chain to continue to see if any

   //of the other loaded mutators want to replace or remove the object. As far

   //as I can tell, there's no reason to mess with bSuperRelevant here.

   return true;

}

defaultproperties
{
    WeaponName=VoidSniperRifle
    AmmoName=VoidAmmo
    WeaponString="VoidSniperRifleV3.VoidSniperRifle"
    AmmoString="VoidSniperRifleV3.VoidAmmo"
}
